@@ -13,7 +13,7 @@
 #   under the License.
 #
 
-import mock
+from unittest import mock
 
 from osc_lib.tests import utils
 
@@ -35,6 +35,8 @@ class TestUndercloudBackup(utils.TestCommand):
         self.cmd = undercloud_backup.BackupUndercloud(self.app, app_args)
         self.app.client_manager.workflow_engine = mock.Mock()
         self.workflow = self.app.client_manager.workflow_engine
+        self.inventory = '/tmp/test_inventory.yaml'
+        self.file = open(self.inventory, 'w').close()
 
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
@@ -144,11 +146,18 @@ class TestUndercloudBackup(utils.TestCommand):
             extra_vars={'sources_path':
                         '/home/stack/,/tmp/foo.yaml'})
 
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
-    def test_undercloud_backup_noargs(self, mock_playbook):
+    def test_undercloud_backup_noargs(self,
+                                      mock_playbook,
+                                      mock_access,
+                                      mock_isfile):
         arglist = []
         verifylist = []
+        mock_isfile.return_value = True
+        mock_access.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -164,13 +173,20 @@ class TestUndercloudBackup(utils.TestCommand):
             extra_vars=None
         )
 
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
-    def test_undercloud_backup_init(self, mock_playbook):
+    def test_undercloud_backup_init(self,
+                                    mock_playbook,
+                                    mock_access,
+                                    mock_isfile):
         arglist = [
             '--init'
         ]
         verifylist = []
+        mock_isfile.return_value = True
+        mock_access.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -186,14 +202,21 @@ class TestUndercloudBackup(utils.TestCommand):
             extra_vars=None
         )
 
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
-    def test_undercloud_backup_init_nfs(self, mock_playbook):
+    def test_undercloud_backup_init_nfs(self,
+                                        mock_playbook,
+                                        mock_access,
+                                        mock_isfile):
         arglist = [
             '--init',
             'nfs'
         ]
         verifylist = []
+        mock_isfile.return_value = True
+        mock_access.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -209,13 +232,20 @@ class TestUndercloudBackup(utils.TestCommand):
             extra_vars=None
         )
 
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
-    def test_undercloud_backup_setup_nfs(self, mock_playbook):
+    def test_undercloud_backup_setup_nfs(self,
+                                         mock_playbook,
+                                         mock_access,
+                                         mock_isfile):
         arglist = [
             '--setup-nfs'
         ]
         verifylist = []
+        mock_isfile.return_value = True
+        mock_access.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -231,13 +261,49 @@ class TestUndercloudBackup(utils.TestCommand):
             extra_vars=None
             )
 
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
-    def test_undercloud_backup_setup_rear(self, mock_playbook):
+    def test_undercloud_backup_db_only(self,
+                                       mock_playbook,
+                                       mock_access,
+                                       mock_isfile):
+        arglist = [
+            '--db-only'
+        ]
+        verifylist = []
+        mock_isfile.return_value = True
+        mock_access.return_value = True
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        mock_playbook.assert_called_once_with(
+            workdir=mock.ANY,
+            playbook='cli-undercloud-db-backup.yaml',
+            inventory=parsed_args.inventory,
+            tags=None,
+            skip_tags=None,
+            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
+            verbosity=3,
+            extra_vars=None
+        )
+
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
+    @mock.patch('tripleoclient.utils.run_ansible_playbook',
+                autospec=True)
+    def test_undercloud_backup_setup_rear(self,
+                                          mock_playbook,
+                                          mock_access,
+                                          mock_isfile):
         arglist = [
             '--setup-rear'
         ]
         verifylist = []
+        mock_isfile.return_value = True
+        mock_access.return_value = True
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -251,24 +317,28 @@ class TestUndercloudBackup(utils.TestCommand):
             playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
             verbosity=3,
             extra_vars=None
-            )
+        )
 
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
     def test_undercloud_backup_setup_rear_extra_vars_inline(self,
-                                                            mock_playbook):
+                                                            mock_playbook,
+                                                            mock_access,
+                                                            mock_isfile):
         arglist = [
             '--setup-rear',
             '--extra-vars',
             '{"tripleo_backup_and_restore_nfs_server": "192.168.24.1"}'
         ]
         verifylist = []
-
+        mock_isfile.return_value = True
+        mock_access.return_value = True
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         extra_vars_dict = {
             'tripleo_backup_and_restore_nfs_server': '192.168.24.1'
         }
-
         self.cmd.take_action(parsed_args)
         mock_playbook.assert_called_once_with(
             workdir=mock.ANY,
@@ -279,7 +349,38 @@ class TestUndercloudBackup(utils.TestCommand):
             playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
             verbosity=3,
             extra_vars=extra_vars_dict
-            )
+        )
+
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
+    @mock.patch('tripleoclient.utils.run_ansible_playbook',
+                autospec=True)
+    def test_undercloud_backup_db_only_with_setup_options(self,
+                                                          mock_playbook,
+                                                          mock_access,
+                                                          mock_isfile):
+        arglist = [
+            '--db-only',
+            '--setup-nfs',
+            '--setup-rear'
+        ]
+        verifylist = []
+        mock_isfile.return_value = True
+        mock_access.return_value = True
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        mock_playbook.assert_called_once_with(
+            workdir=mock.ANY,
+            playbook='cli-undercloud-db-backup.yaml',
+            inventory=parsed_args.inventory,
+            tags=None,
+            skip_tags=None,
+            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
+            verbosity=3,
+            extra_vars=None
+        )
 
     @mock.patch('tripleoclient.utils.run_ansible_playbook', autospec=True)
     def test_undercloud_backup_setup_nfs_rear_with_inventory(self,
@@ -288,14 +389,12 @@ class TestUndercloudBackup(utils.TestCommand):
             '--setup-nfs',
             '--setup-rear',
             '--inventory',
-            '/tmp/test_inventory.yaml'
+            self.inventory
         ]
         verifylist = []
-
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         self.cmd.take_action(parsed_args)
-
         calls = [call(workdir=mock.ANY,
                       playbook='prepare-nfs-backup.yaml',
                       inventory=parsed_args.inventory,
@@ -315,16 +414,22 @@ class TestUndercloudBackup(utils.TestCommand):
 
         mock_playbook.assert_has_calls(calls)
 
+    @mock.patch('os.path.isfile')
+    @mock.patch('os.access')
     @mock.patch('tripleoclient.utils.run_ansible_playbook',
                 autospec=True)
-    def test_undercloud_backup_setup_nfs_with_extra_vars(self, mock_playbook):
+    def test_undercloud_backup_setup_nfs_with_extra_vars(self,
+                                                         mock_playbook,
+                                                         mock_access,
+                                                         mock_isfile):
         arglist = [
             '--setup-nfs',
             '--extra-vars',
             '/tmp/test_vars.yaml'
         ]
         verifylist = []
-
+        mock_isfile.return_value = True
+        mock_access.return_value = True
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         self.cmd.take_action(parsed_args)
@@ -344,10 +449,9 @@ class TestUndercloudBackup(utils.TestCommand):
     def test_undercloud_backup_inventory(self, mock_playbook):
         arglist = [
             '--inventory',
-            '/tmp/test_inventory.yaml'
+            self.inventory
         ]
         verifylist = []
-
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         self.cmd.take_action(parsed_args)
@@ -361,3 +465,39 @@ class TestUndercloudBackup(utils.TestCommand):
             verbosity=3,
             extra_vars=None
         )
+
+    @mock.patch('tripleoclient.utils.run_ansible_playbook',
+                autospec=True)
+    def test_undercloud_backup_no_inventory(self, mock_playbook):
+        arglist = [
+            '--inventory',
+            '/tmp/no_inventory.yaml'
+        ]
+        verifylist = []
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaisesRegex(
+            RuntimeError,
+            'The inventory file',
+            self.cmd.take_action,
+            parsed_args)
+
+    @mock.patch('os.access')
+    @mock.patch('tripleoclient.utils.run_ansible_playbook',
+                autospec=True)
+    def test_undercloud_backup_no_readable_inventory(self,
+                                                     mock_playbook,
+                                                     mock_access):
+        arglist = [
+            '--inventory',
+            self.inventory
+        ]
+        verifylist = []
+        mock_access.return_value = False
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaisesRegex(
+            RuntimeError,
+            'The inventory file',
+            self.cmd.take_action,
+            parsed_args)
