@@ -16,17 +16,26 @@
 import collections
 import fixtures
 import json
-import mock
 import os
 import tempfile
+from unittest import mock
 
+import openstack
 from osc_lib.tests import utils as test_utils
 
 from tripleoclient import constants
 from tripleoclient.tests.v2.overcloud_node import fakes
 from tripleoclient.v2 import overcloud_node
+from tripleoclient.workflows import tripleo_baremetal as tb
 
 
+@mock.patch('tripleoclient.utils.run_ansible_playbook',
+            autospec=True)
+@mock.patch.object(openstack.baremetal.v1._proxy, 'Proxy', autospec=True,
+                   name="mock_bm")
+@mock.patch('openstack.config', autospec=True, name='mock_conf')
+@mock.patch('openstack.connect', autospec=True, name='mock_connect')
+@mock.patch.object(openstack.connection, 'Connection', autospec=True)
 class TestImportNode(fakes.TestOvercloudNode):
 
     def setUp(self):
@@ -49,6 +58,18 @@ class TestImportNode(fakes.TestOvercloudNode):
             ]
         }]
 
+        self.fake_baremetal_node = fakes.make_fake_machine(
+            machine_name='node1',
+            machine_id='4e540e11-1366-4b57-85d5-319d168d98a1',
+            provision_state='manageable',
+            is_maintenance=False
+        )
+        self.fake_baremetal_node2 = fakes.make_fake_machine(
+            machine_name='node2',
+            machine_id='9070e42d-1ad7-4bd0-b868-5418bc9c7176',
+            provision_state='manageable',
+            is_maintenance=False
+        )
         self.json_file = tempfile.NamedTemporaryFile(
             mode='w', delete=False, suffix='.json')
         json.dump(self.nodes_list, self.json_file)
@@ -72,18 +93,24 @@ class TestImportNode(fakes.TestOvercloudNode):
                                               for i in ('agent.kernel',
                                                         'agent.ramdisk')]))
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_import_only(self, mock_playbook):
+    def test_import_only(self,
+                         mock_conn,
+                         mock_connect,
+                         mock_conf,
+                         mock_bm,
+                         mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         [self.json_file.name],
                                         [('introspect', False),
                                          ('provide', False)])
         self.cmd.take_action(parsed_args)
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_import_and_introspect(self, mock_playbook):
+    def test_import_and_introspect(self,
+                                   mock_conn,
+                                   mock_connect,
+                                   mock_conf,
+                                   mock_bm,
+                                   mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         [self.json_file.name,
                                          '--introspect'],
@@ -103,25 +130,45 @@ class TestImportNode(fakes.TestOvercloudNode):
             }
         )
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_import_and_provide(self, mock_playbook):
+    def test_import_and_provide(self,
+                                mock_conn,
+                                mock_connect,
+                                mock_conf,
+                                mock_bm,
+                                mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         [self.json_file.name,
                                          '--provide'],
                                         [('introspect', False),
                                          ('provide', True)])
+        mock_conn.return_value = mock_bm
+        mock_bm.baremetal = mock_bm
+        mock_bm.baremetal_introspection = mock_bm
+        mock_bm.baremetal.get_node.side_effect = [
+            self.fake_baremetal_node,
+            self.fake_baremetal_node2]
+
         self.cmd.take_action(parsed_args)
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_import_and_introspect_and_provide(self, mock_playbook):
+    def test_import_and_introspect_and_provide(self,
+                                               mock_conn,
+                                               mock_connect,
+                                               mock_conf,
+                                               mock_bm,
+                                               mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         [self.json_file.name,
                                          '--introspect',
                                          '--provide'],
                                         [('introspect', True),
                                          ('provide', True)])
+        mock_conn.return_value = mock_bm
+        mock_bm.baremetal = mock_bm
+        mock_bm.baremetal_introspection = mock_bm
+        mock_bm.baremetal.get_node.side_effect = [
+            self.fake_baremetal_node,
+            self.fake_baremetal_node2]
+
         self.cmd.take_action(parsed_args)
         mock_playbook.assert_called_with(
             workdir=mock.ANY,
@@ -130,22 +177,30 @@ class TestImportNode(fakes.TestOvercloudNode):
             playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
             verbosity=mock.ANY,
             extra_vars={
-                'node_uuids': ['MOCK_NODE_UUID']
+                'node_uuids': ['MOCK_NODE_UUID'],
+                'run_validations': False,
+                'concurrency': 20
             }
         )
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_import_with_netboot(self, mock_playbook):
+    def test_import_with_netboot(self,
+                                 mock_conn,
+                                 mock_connect,
+                                 mock_conf,
+                                 mock_bm,
+                                 mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         [self.json_file.name,
                                          '--no-deploy-image'],
                                         [('no_deploy_image', True)])
         self.cmd.take_action(parsed_args)
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_import_with_no_deployed_image(self, mock_playbook):
+    def test_import_with_no_deployed_image(self,
+                                           mock_conn,
+                                           mock_connect,
+                                           mock_conf,
+                                           mock_bm,
+                                           mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         [self.json_file.name,
                                          '--instance-boot-option',
@@ -154,16 +209,37 @@ class TestImportNode(fakes.TestOvercloudNode):
         self.cmd.take_action(parsed_args)
 
 
+@mock.patch('tripleoclient.utils.run_ansible_playbook',
+            autospec=True)
+@mock.patch.object(openstack.baremetal.v1._proxy, 'Proxy', autospec=True,
+                   name="mock_bm")
+@mock.patch('openstack.config', autospec=True, name='mock_conf')
+@mock.patch('openstack.connect', autospec=True, name='mock_connect')
+@mock.patch.object(openstack.connection, 'Connection', autospec=True)
 class TestIntrospectNode(fakes.TestOvercloudNode):
 
     def setUp(self):
         super(TestIntrospectNode, self).setUp()
         # Get the command object to test
         self.cmd = overcloud_node.IntrospectNode(self.app, None)
+        self.fake_baremetal_node = fakes.make_fake_machine(
+            machine_name='node1',
+            machine_id='4e540e11-1366-4b57-85d5-319d168d98a1',
+            provision_state='manageable',
+            is_maintenance=False
+        )
+        self.fake_baremetal_node2 = fakes.make_fake_machine(
+            machine_name='node2',
+            machine_id='9070e42d-1ad7-4bd0-b868-5418bc9c7176',
+            provision_state='manageable',
+            is_maintenance=False
+        )
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
     def test_introspect_all_manageable_nodes_without_provide(self,
+                                                             mock_conn,
+                                                             mock_connect,
+                                                             mock_conf,
+                                                             mock_bm,
                                                              mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         ['--all-manageable'],
@@ -185,29 +261,54 @@ class TestIntrospectNode(fakes.TestOvercloudNode):
             }
         )
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
     def test_introspect_all_manageable_nodes_with_provide(self,
+                                                          mock_conn,
+                                                          mock_connect,
+                                                          mock_conf,
+                                                          mock_bm,
                                                           mock_playbook):
         parsed_args = self.check_parser(self.cmd,
                                         ['--all-manageable', '--provide'],
                                         [('all_manageable', True),
                                          ('provide', True)])
+        tb.TripleoProvide.provide = mock.MagicMock()
+        mock_conn.return_value = mock_bm
+        mock_bm.baremetal = mock_bm
+        mock_bm.baremetal.nodes.side_effect = [
+            iter([self.fake_baremetal_node,
+                  self.fake_baremetal_node2]),
+            iter([self.fake_baremetal_node,
+                  self.fake_baremetal_node2])
+            ]
+
+        expected_nodes = ['4e540e11-1366-4b57-85d5-319d168d98a1',
+                          '9070e42d-1ad7-4bd0-b868-5418bc9c7176']
         self.cmd.take_action(parsed_args)
         mock_playbook.assert_called_with(
             workdir=mock.ANY,
-            playbook='cli-overcloud-node-provide.yaml',
+            playbook='cli-baremetal-introspect.yaml',
             inventory=mock.ANY,
             playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
             verbosity=mock.ANY,
             extra_vars={
-                'node_uuids': []
+                'node_uuids': [],
+                'run_validations': False,
+                'concurrency': 20,
+                'node_timeout': 1200,
+                'max_retries': 1,
+                'retry_timeout': 120,
             }
         )
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_introspect_nodes_without_provide(self, mock_playbook):
+        tb.TripleoProvide.provide.assert_called_with(
+            expected_nodes)
+
+    def test_introspect_nodes_without_provide(self,
+                                              mock_conn,
+                                              mock_connect,
+                                              mock_conf,
+                                              mock_bm,
+                                              mock_playbook):
         nodes = ['node_uuid1', 'node_uuid2']
         parsed_args = self.check_parser(self.cmd,
                                         nodes,
@@ -229,37 +330,48 @@ class TestIntrospectNode(fakes.TestOvercloudNode):
             }
         )
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_introspect_nodes_with_provide(self, mock_playbook):
-        nodes = ['node_uuid1', 'node_uuid2']
+    def test_introspect_nodes_with_provide(self,
+                                           mock_conn,
+                                           mock_connect,
+                                           mock_conf,
+                                           mock_bm,
+                                           mock_playbook):
+        nodes = ['node1', 'node2']
         argslist = nodes + ['--provide']
         parsed_args = self.check_parser(self.cmd,
                                         argslist,
                                         [('node_uuids', nodes),
                                          ('provide', True)])
+        tb.TripleoProvide.provide = mock.MagicMock()
+        mock_conn.return_value = mock_bm
+        mock_bm.baremetal = mock_bm
+        mock_bm.baremetal_introspection = mock_bm
+        mock_bm.baremetal.get_node.side_effect = [
+            self.fake_baremetal_node,
+            self.fake_baremetal_node2]
+
         self.cmd.take_action(parsed_args)
-        mock_playbook.assert_called_with(
-            workdir=mock.ANY,
-            playbook='cli-overcloud-node-provide.yaml',
-            inventory=mock.ANY,
-            playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
-            verbosity=mock.ANY,
-            extra_vars={
-                'node_uuids': nodes
-            }
+
+        tb.TripleoProvide.provide.assert_called_with(
+            nodes=nodes
         )
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_introspect_no_node_or_flag_specified(self, mock_playbook):
+    def test_introspect_no_node_or_flag_specified(self,
+                                                  mock_conn,
+                                                  mock_connect,
+                                                  mock_conf,
+                                                  mock_bm,
+                                                  mock_playbook):
         self.assertRaises(test_utils.ParserException,
                           self.check_parser,
                           self.cmd, [], [])
 
-    @mock.patch('tripleoclient.utils.run_ansible_playbook',
-                autospec=True)
-    def test_introspect_uuids_and_all_both_specified(self, mock_playbook):
+    def test_introspect_uuids_and_all_both_specified(self,
+                                                     mock_conn,
+                                                     mock_connect,
+                                                     mock_conf,
+                                                     mock_bm,
+                                                     mock_playbook):
         argslist = ['node_id1', 'node_id2', '--all-manageable']
         verifylist = [('node_uuids', ['node_id1', 'node_id2']),
                       ('all_manageable', True)]
@@ -267,7 +379,12 @@ class TestIntrospectNode(fakes.TestOvercloudNode):
                           self.check_parser,
                           self.cmd, argslist, verifylist)
 
-    def _check_introspect_all_manageable(self, parsed_args, provide=False):
+    def _check_introspect_all_manageable(self, parsed_args,
+                                         mock_conn,
+                                         mock_connect,
+                                         mock_conf,
+                                         mock_bm,
+                                         mock_playbook, provide=False):
         self.cmd.take_action(parsed_args)
 
         call_list = [mock.call(
@@ -285,7 +402,12 @@ class TestIntrospectNode(fakes.TestOvercloudNode):
         self.assertEqual(self.workflow.executions.create.call_count,
                          2 if provide else 1)
 
-    def _check_introspect_nodes(self, parsed_args, nodes, provide=False):
+    def _check_introspect_nodes(self, parsed_args, nodes,
+                                mock_conn,
+                                mock_connect,
+                                mock_conf,
+                                mock_bm,
+                                mock_playbook, provide=False):
         self.cmd.take_action(parsed_args)
 
         call_list = [mock.call(
@@ -337,10 +459,12 @@ class TestProvisionNode(fakes.TestOvercloudNode):
 
                     argslist = ['--output', outp.name,
                                 '--overcloud-ssh-key', keyf.name,
+                                '--yes',
                                 inp.name]
                     verifylist = [('input', inp.name),
                                   ('output', outp.name),
-                                  ('overcloud_ssh_key', keyf.name)]
+                                  ('overcloud_ssh_key', keyf.name),
+                                  ('yes', True)]
 
                     parsed_args = self.check_parser(self.cmd,
                                                     argslist, verifylist)
@@ -363,6 +487,7 @@ class TestProvisionNode(fakes.TestOvercloudNode):
                 'configure_networking': False,
                 'working_dir': mock.ANY,
                 'templates': constants.TRIPLEO_HEAT_TEMPLATES,
+                'overwrite': True,
             },
             inventory='localhost,',
             playbook='cli-overcloud-node-provision.yaml',
@@ -441,7 +566,7 @@ class TestUnprovisionNode(fakes.TestOvercloudNode):
                     ],
                     'all': True,
                     'prompt': False,
-                    'manage_network_ports': False,
+                    'manage_network_ports': True,
                 },
                 inventory='localhost,',
                 playbook='cli-overcloud-node-unprovision.yaml',
