@@ -28,7 +28,6 @@ import yaml
 
 from tripleoclient import command
 from tripleoclient import constants
-from tripleoclient import exceptions
 from tripleoclient import utils as oooutils
 from tripleoclient.workflows import baremetal
 
@@ -39,6 +38,7 @@ from tripleoclient.v1.overcloud_node import ConfigureNode  # noqa
 from tripleoclient.v1.overcloud_node import DeleteNode  # noqa
 from tripleoclient.v1.overcloud_node import DiscoverNode  # noqa
 from tripleoclient.v1.overcloud_node import ProvideNode  # noqa
+from tripleoclient.workflows import tripleo_baremetal as tb
 
 
 class ImportNode(command.Command):
@@ -88,6 +88,9 @@ class ImportNode(command.Command):
                             default=20,
                             help=_('Maximum number of nodes to introspect at '
                                    'once.'))
+        parser.add_argument('--verbosity', type=int,
+                            default=1,
+                            help=_('Print debug logs during execution'))
         parser.add_argument('env_file', type=argparse.FileType('r'))
         return parser
 
@@ -133,10 +136,8 @@ class ImportNode(command.Command):
                 )
 
         if parsed_args.provide:
-            baremetal.provide(
-                verbosity=oooutils.playbook_verbosity(self=self),
-                node_uuids=nodes_uuids
-            )
+            provide = tb.TripleoProvide(verbosity=parsed_args.verbosity)
+            provide.provide(nodes=nodes_uuids)
 
 
 class IntrospectNode(command.Command):
@@ -180,6 +181,9 @@ class IntrospectNode(command.Command):
                             default=120,
                             help=_('Maximum timeout between introspection'
                                    'retries'))
+        parser.add_argument('--verbosity', type=int,
+                            default=1,
+                            help=_('Print debug logs during execution'))
         return parser
 
     def take_action(self, parsed_args):
@@ -210,16 +214,13 @@ class IntrospectNode(command.Command):
         # NOTE(cloudnull): This is using the old provide function, in a future
         #                  release this may be ported to a standalone playbook
         if parsed_args.provide:
+            provide = tb.TripleoProvide(verbosity=parsed_args.verbosity)
             if parsed_args.node_uuids:
-                baremetal.provide(
-                    node_uuids=parsed_args.node_uuids,
-                    verbosity=oooutils.playbook_verbosity(self=self)
+                provide.provide(
+                    nodes=parsed_args.node_uuids,
                 )
             else:
-                baremetal.provide_manageable_nodes(
-                    clients=self.app.client_manager,
-                    verbosity=oooutils.playbook_verbosity(self=self)
-                )
+                provide.provide_manageable_nodes()
 
 
 class ProvisionNode(command.Command):
@@ -236,6 +237,9 @@ class ProvisionNode(command.Command):
         parser.add_argument('-o', '--output',
                             default='baremetal_environment.yaml',
                             help=_('The output environment file path'))
+        parser.add_argument('-y', '--yes', default=False, action='store_true',
+                            help=_('Skip yes/no prompt for existing files '
+                                   '(assume yes).'))
         parser.add_argument('--stack', dest='stack',
                             help=_('Name or ID of heat stack '
                                    '(default=Env: OVERCLOUD_STACK_NAME)'),
@@ -282,14 +286,6 @@ class ProvisionNode(command.Command):
 
         return parser
 
-    def _validate_playbook(self, playbook_path):
-        if not os.path.exists(playbook_path):
-            raise exceptions.PlaybookNotFound(
-                'Playbook file {} not found.'.format(playbook_path))
-        if not os.path.isfile(playbook_path):
-            raise exceptions.InvalidPlaybook(
-                'Playbook {} is not a file.'.format(playbook_path))
-
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)" % parsed_args)
 
@@ -298,6 +294,18 @@ class ProvisionNode(command.Command):
                              'This option is no longer used, network ports '
                              'are always managed.')
 
+        output_path = os.path.abspath(parsed_args.output)
+
+        overwrite = parsed_args.yes
+        if (os.path.exists(output_path) and not overwrite
+                and not oooutils.prompt_user_for_confirmation(
+                    'Overwrite existing file %s [y/N]?' % parsed_args.output,
+                    self.log)):
+            raise oscexc.CommandError("Will not overwrite existing file:"
+                                      " %s" % parsed_args.output)
+        else:
+            overwrite = True
+
         if not parsed_args.working_dir:
             working_dir = oooutils.get_default_working_dir(
                 parsed_args.stack)
@@ -316,8 +324,6 @@ class ProvisionNode(command.Command):
         with open('{}.pub'.format(key), 'rt') as fp:
             ssh_key = fp.read()
 
-        output_path = os.path.abspath(parsed_args.output)
-
         extra_vars = {
             "stack_name": parsed_args.stack,
             "baremetal_deployment": roles,
@@ -331,6 +337,7 @@ class ProvisionNode(command.Command):
             "configure_networking": parsed_args.network_config,
             "working_dir": working_dir,
             "templates": parsed_args.templates,
+            "overwrite": overwrite,
         }
 
         with oooutils.TempDirs() as tmp:
@@ -417,14 +424,36 @@ class UnprovisionNode(command.Command):
                 )
                 with open(unprovision_confirm) as f:
                     to_unprovision = json.load(f)
-                    if not to_unprovision:
+
+                    # (TODO: slagle) unprovision_confirm was previously a list,
+                    # but was switched to a dict so that network ports for
+                    # pre_provisioned nodes can also be confirmed for
+                    # unprovisioning. Check the data structure for backwards
+                    # compatibility, When the tripleo-ansible patch is merged,
+                    # this check can be removed.
+                    if isinstance(to_unprovision, dict):
+                        instances = to_unprovision.get('instances')
+                        pre_provisioned = to_unprovision.get('pre_provisioned')
+                    else:
+                        instances = to_unprovision
+                        pre_provisioned = None
+
+                    print()
+                    if not (instances or pre_provisioned):
                         print('Nothing to unprovision, exiting')
                         return
-                    self._print_nodes(to_unprovision)
+                    print("The following nodes will be unprovisioned:")
+                    self._print_nodes(instances)
+                    print()
+                    if pre_provisioned:
+                        print("The following pre-provisioned nodes will "
+                              "have network ports unprovisioned:")
+                        self._print_nodes(pre_provisioned)
+                        print()
 
                 confirm = oooutils.prompt_user_for_confirmation(
                     message=_("Are you sure you want to unprovision these %s "
-                              "nodes [y/N]? ") % parsed_args.stack,
+                              "nodes and ports [y/N]? ") % parsed_args.stack,
                     logger=self.log)
                 if not confirm:
                     raise oscexc.CommandError("Action not confirmed, exiting.")
@@ -440,7 +469,7 @@ class UnprovisionNode(command.Command):
                     "baremetal_deployment": roles,
                     "all": parsed_args.all,
                     "prompt": False,
-                    "manage_network_ports": parsed_args.network_ports,
+                    "manage_network_ports": True,
                 }
             )
 
